function ControlApp() {
    this.spinner = new Spinner();
    this.leaderClient = null;
    this.followerClient = null;
    this.alertElement = null;
    this.alertTimeout = null;
}

ControlApp.prototype = {
    initilialize: function() {
        this.spinner.initialize("spinner");

        // Enable fullscreen on double click
        var toplevel = document.getElementById("toplevel");
        toplevel.addEventListener("dblclick", () => this.requestFullScreen());

        // Initialize error dialog
        this.alertElement = document.getElementById("alert-text");

        // Load config
        this.getApplicationConfiguration()
            .then((config) => {
                this.createVideoContainer(config);
                this.initControlBar();
                this.createClients(config);
            })
            .catch(() => this.displayError("Erreur lors du chargement de la configuration (config.json)"));
    },

    createClients: function(config) {
        var timeout = 5000;
        var user = "user";

        this.leaderClient = IDAL.getClient(config.leader ? config.leader : window.location.hostname, user, config.password, { timeout: timeout });
        this.followerClient = IDAL.getClient(config.follower, user, config.password, { timeout: timeout });
    },

    createVideoContainer: function(config) {
        var videos = config.videos;
        var container = document.getElementById("video-grid");

        if (videos && videos.length) {
            for (var i = 0 ; i < videos.length ; i++) {
                var current = videos[i];
                container.appendChild(this.createVideoItem(current));
            }
        } else {
            this.displayError("Aucune vidéo définie !");
        }
    },

    createVideoItem: function(params) {
        var cell = document.createElement("div");
        cell.className = "video-cell";

        var button = document.createElement("div");
        button.className = "video-button";
        button.style.backgroundImage = "url(images/" + params.image + ")"

        cell.appendChild(button);

        var title = document.createElement("span");
        title.innerText = params.title;
        button.appendChild(title);

        var folder = params.folder;
        button.addEventListener("click", () => this.onVideoClick(folder));

        return cell;
    },

    initControlBar: function() {
        var button = document.getElementById("toggle-pause");
        button.addEventListener("click", () => this.onTogglePauseClick());


        button = document.getElementById("home");
        button.addEventListener("click", () => this.onHomeClick());

        button = document.getElementById("toggle-standby");
        button.addEventListener("click", () => this.onToggleStandbyClick());
    },

    sendPostAll: function(api) {
        return this.spinner.showSpinner(Promise.all(
            [
                this.leaderClient.post(api),
                this.followerClient.post(api)
            ]
        ));
    },

    onTogglePauseClick: function() {
        console.log("Toggle pause");

        // Not using /api/player/toggle-pause directly
        // to ensure slave/follower consistency
        var promise = this.leaderClient.get("/api/system/state/playbackengine")
            .catch((e) => {
                throw "Impossible d'obtenir l'état du lecteur";
            })
            .then((response) => {
                if (response && response.data) {
                    var api;
                    if (response.data.state == 'playing') {
                        api = "/api/player/pause";
                    } else {
                        api = "/api/player/resume";
                    }
                    return this.sendPostAll(api)
                        .catch((e) => {
                            throw "Erreur lors de l'envoi de la commande de pause";
                        });
                } else {
                    throw "Impossible d'obtenir l'état du lecteur";
                }
            })
            .catch((e) => this.displayError(e));

        this.spinner.showSpinner(promise);
    },

    onHomeClick: function() {
        console.log("Go to home");
        this.sendPostAll("/api/player/stop")
            .catch((e) => this.displayError("Erreur lors du retour à l'accueil"));
    },

    onToggleStandbyClick: function() {
        console.log("Toggle standby");

        // Not using /api/power/toggle-standby directly
        // to ensure slave/follower consistency
        var promise = this.leaderClient.get("/api/system/state/power")
            .catch((e) => {
                throw "Impossible d'obtenir l'état du lecteur";
            })
            .then((response) => {
                if (response && response.data) {
                    var api;
                    if (response.data.standby) {
                        api = "/api/power/wakeup";
                    } else {
                        api = "/api/power/standby";
                    }
                    return this.sendPostAll(api)
                        .catch((e) => {
                            throw "Erreur lors de l'envoi de la commande de mise en veille / réveil";
                        });
                } else {
                    throw "Impossible d'obtenir l'état du lecteur";
                }
            })
            .catch((e) => this.displayError(e));

        this.spinner.showSpinner(promise);
    },

    onVideoClick: function(folder) {
        console.log("Play folder " + folder);
        this.leaderClient.post("/api/player/play-folder?folder=" + folder)
            .catch(() => this.displayError("Erreur lors de l'envoi de la commande de lecture du dossier " + folder));
    },

    getApplicationConfiguration: function() {
        return new Promise(function(resolve, reject) {
            var xhr = new XMLHttpRequest();
            xhr.addEventListener("readystatechange", function() {
                if(this.readyState === 4) {
                    if (xhr.status === 200) {
                        try {
                            var response = JSON.parse(this.responseText);
                            resolve(response);
                        } catch(e) {
                            reject(e);
                        }
                    } else {
                        reject("Invalid HTTP response status " + xhr.status);
                    }
                }
            });
            console.log("Getting config.json (you can edit this file with your settings)...");
            xhr.open("GET", "config.json");
            xhr.send();
        });
    },

    /**
     * Display an error message during a short period of time
     * @param {*} message
     */
    displayError: function(message) {
        console.log("Erreur: " + message);
        if (this.alertTimeout !== null) {
            window.clearTimeout(this.alertTimeout);
            this.alertTimeout = null;
        }

        this.alertElement.classList.add("show");
        this.alertElement.innerText = message;

        this.alertTimeout = window.setTimeout(() => {
            this.alertElement.classList.remove("show");
            this.alertTimeout = null;
        }, 4000);
    },

    requestFullScreen: function() {
        var doc = window.document;
        if (!doc.fullscreenElement && !doc.mozFullScreenElement && !doc.webkitFullscreenElement && !doc.msFullscreenElement) {
            const docEl = doc.documentElement;
            const requestFullScreen = docEl.requestFullscreen ||
                docEl.mozRequestFullScreen ||
                docEl.webkitRequestFullScreen ||
                docEl.msRequestFullscreen;
            if (requestFullScreen) {
                // Fullscreen request can only be requested by user gesture, so may
                // throw an exception...
                try {
                    requestFullScreen.call(docEl);
                } catch(e) {

                }
            }
        }
    },
};



window.addEventListener('load', function() {
    var app = new ControlApp();
    app.initilialize();
});